import { FC, useCallback, useEffect } from 'react'
import { FieldType, FieldTypes, fieldTypeOptions, fieldTypesWithValues } from '../../types/schema'
import { useObjectStateWrapper } from '../../hooks/useObjectState';
import { FieldTypeInput } from './field-type-input';
import { InputBinder } from '../../types/inputBinder';
import { ValueField } from './value-field';

interface IProps {
  update: (arg: FieldType) => void;
  field: FieldType;
  fieldName: string;
}

export const FieldEditor: FC<IProps> = ({ update, field, fieldName }) => {
  const { update: updateField, bindProperty, bindPropertyCheck } = useObjectStateWrapper(field, (e) => update(typeof e === 'function' ? e(field) : e))

  const shouldShowValueField = useCallback(() => fieldTypesWithValues.includes(field.type) || field.isConstant, [field.isConstant, field.type]);

  useEffect(() => {
    console.log(field.value);
  }, [field])

  return (
    <li className="odd:bg-black/50 flex gap-4 items-center p-2">
      <p>{fieldName}</p>
      <select className="capitalize" {...bindProperty('type')}>
        {fieldTypeOptions.map(o => (
          <option className="capitalize" value={FieldTypes[o]}>{o}</option>
        ))}
      </select>
      {shouldShowValueField() && (
        // <>
        //   {field.type === FieldTypes.dice && (
        //     <label>
        //       Sides:&nbsp;
        //       <select {...bindProperty('value')}>
        //         {diceSides.map(d => (
        //           <option value={'d' + d}>{d}</option>
        //         ))}
        //       </select>
        //     </label>
        //   )}
        //   {field.type === FieldTypes.type && (
        //     <FieldTypeInput bind={bindProperty('value')} />

        //   )}
        // </>
        <ValueField type={field.type} bind={bindProperty('value')} />
      )}
      <label><input type="checkbox" {...bindPropertyCheck('isConstant')} /> Is constant</label>
    </li>
  )
}
