import { FC, useCallback } from 'react'
import { FCC } from '../../types'
import { FieldType, FieldTypes, TypeType } from '../../types/schema'
import { useObjectState } from '../../hooks/useObjectState';
import { useInput } from '../../hooks/useInput';
import { FieldEditor } from './field-editor';

interface IProps {
  name: string;
  saveType: (arg0: string, arg1: TypeType) => void;
}

const constantProperties = ['metadata'];

export const TypeEditor: FCC<IProps> = ({ saveType, name }) => {
  const { update: updateType, reset: resetType, state: type } = useObjectState<TypeType>({});

  const { value: propertyName, setValue: setPropertyName, bind: bindPropertyName, reset: resetPropertyName } = useInput('');

  const save = () => {
    saveType(name, type);
    resetType();
  }

  const addField = useCallback(() => {
    updateType({
      [propertyName]: {
        type: FieldTypes.number,
        value: '',
        isConstant: false,
        limit: 1,
      }
    })
  }, [propertyName, updateType])

  const updateField = useCallback((k: keyof typeof type) => (field: FieldType) => {
    updateType({ [k]: field })
  }, [updateType])

  return (
    <div>
      <p className="subheader">Creating type "{name}"</p>
      <input type="text" {...bindPropertyName} />
      <button disabled={!propertyName} onClick={addField}>Add Field</button>
      <ul>
        {Object.entries(type).filter(([k]) => !constantProperties.includes(k)).map(([key, value]) => (
          <FieldEditor field={value} update={updateField(key)} fieldName={key} />
        ))}
      </ul>
    </div>
  )
}